# Man-Yong Lee <yong@linuxkorea.co.kr>
# Last update: 2001-04-19

import codecs

from encodings.korean import euc2ucs, ucs2euc
# on demand
#from encodings.korean import hanja2ucs, ucs2hanja

class Codec(codecs.Codec):
    # Unicode to character buffer
    def encode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise UnicodeError, "unknown error handling"

     	euc_chars = ucs2euc.mapping
	misc_chars = {} # Misc. (Special characters)
	chinese_chars = {} # Hanja (Chinese characters)

        s = ''
        for c in data:
            if c < u'\u0080':
                s = s + c.encode("ascii")
            else:
                if euc_chars.has_key(c):
                    s = s + euc_chars[c]
		else:
		    # Rationale:
		    # These case does not happen frequently :)
		    if not misc_chars:
		        from encodings.korean import ucs2hanja
			from encodings.korean import ucs2misc

			misc_chars = ucs2misc.mapping
			chinese_chars = ucs2hanja.mapping

		    if misc_chars.has_key(c):
		        s = s + misc_chars[c]
		    elif chinese_chars.has_key(c):
		        s = s + chinese_chars[c]
                    elif errors == 'replace':
                        s = s + '\xa2\xae' # EUC-KR
                    elif errors == 'strict':
                        raise UnicodeError, "cannot map \\u%04x to EUC-KR" % ord(c)
        return (s, len(s))

    # character buffer to Unicode
    def decode(self, data, errors='strict',
               supported_errors=('strict', 'ignore', 'replace')):
        if errors not in supported_errors:
            raise UnicodeError, "unknown error handling"

	euc_chars = euc2ucs.mapping
	misc_chars = {}
	chinese_chars = {}

        s = u''
        data = str(data) # character buffer compatible object
        size = len(data)
        p = 0
        while p < size:
            if data[p] < "\x80":
                s = s + unicode(data[p], "ascii")
                p = p + 1
            else:
                c = data[p:p+2]
                p = p + 2

                if euc_chars.has_key(c):
                    s = s + euc_chars[c]
		else:
		    if not misc_chars:
		        from encodings.korean import hanja2ucs
			from encodings.korean import misc2ucs

			misc_chars = misc2ucs.mapping
			chinese_chars = hanja2ucs.mapping

		    if misc_chars.has_key(c):
                        s = s + misc_chars[c]
		    elif chinese_chars.has_key(c):
                        s = s + chinese_chars[c]
                    elif errors == 'replace':
                        s = s + u'\uFFFD' # REPLACEMENT CHARACTER
                    elif errors == 'strict':
                        raise UnicodeError, "cannot map EUC-KR \\x%02x%02x to Unicode" % (ord(c[0]), ord(c[1]))
        return (s, p)

class StreamWriter(Codec, codecs.StreamWriter):
    pass

class StreamReader(Codec, codecs.StreamReader):
    def __init__(self, stream, errors='strict'):
        codecs.StreamReader.__init__(self, stream, errors)
        self.data = ''
    def read(self, size=-1):
        if size < 0:
            data = self.data + self.stream.read()
            self.data = ''
        else:
            data = self.data + self.stream.read(size - len(self.data))
            size = len(data)
            p = 0
            while p < size:
                if data[p] < "\x80":
                    p = p + 1
                elif p + 2 <= size:
                    p = p + 2
                else:
                    break
            data, self.data = data[:p], data[p:]
        return self.decode(data)[0]
    def readline(self, size=None):
        if size is None:
            data = self.data + self.stream.readline()
        else:
            data = self.data + self.stream.readline(size - len(self.data))
        self.data = ''
        return self.decode(data)[0]
    def reset(self):
        self.data = ''

### encodings module API

def getregentry():

    return (Codec().encode,Codec().decode,StreamReader,StreamWriter)

def getaliases():

    return ('ksc5601', )

